/* heston.h: header for analytic solution for HESTON's model
 *
 * author: Gunter Winkler (gw) 
 *         gunter.winkler@mathematik.tu-chemnitz.de
 *
 * changes:
 *	gw	2001-08-07	new
 *
 */

#ifndef __INC_HESTON__
#define __INC_HESTON__

#include <iostream>
#include <cmath>
#include <complex>

#include "iqlvms.h"

typedef double				DOUBLE;
typedef std::complex<DOUBLE>	heston_complex;

/*
 *  defines
 */

#ifndef NDEBUG
# ifndef DEBUGLEVEL
#  define DEBUGLEVEL 10
# endif
# define DEBUG_HESTON(lvl,x)	\
		if ((DEBUGLEVEL)>=(lvl)) { x }
#else
# define DEBUG_HESTON(lvl,x)	
#endif

/*
 *	constants
 */


/*
 *	types
 */

struct OPTIONDATA {
	DOUBLE	strike;
	DOUBLE	spot;
	DOUBLE	time;		/* time to maturity */
	DOUBLE	putcall;	/* 1 = call, -1 = put */
};

struct MARKETDATA {
	DOUBLE	rd, rf; 	/* (logarithmic) domestic/foreign interest rates */
	DOUBLE	rho, kappa, theta, xi;	/* stochastic volatility parameters */
	DOUBLE	lambda; 	/* price of volatility risk */
};

/*
 *	functions
 */

struct intdata {
  int N;       /* number of integration points */
  double * x;  /* coordinates */
  double * w;  /* weights */
  /* \int f(x) dx \approx \sum_{i=0}^{N-1} w[i] * f(x[i]) */ 
};

typedef intdata *p_intdata;

/*
 *	formula for vanilla call (same as Black-Scholes)
 */
double	Heston_Call( const DOUBLE spot,
					 const DOUBLE v,
					 const DOUBLE tau,
					 const DOUBLE strike,
					 MARKETDATA market,
					 p_intdata *pdata = NULL);

/* 
 * compute the eigenvalues of a size-by-size symmetric tridiagonal matrix
 *
 * nev      : number components of eigenvectors
 * size     : size of input matrix
 * maindiag : main diagonal (vector with size components)
 * subdiag  : sub/super diagonal (vector with size components, 
 *            first element will be ignored)
 * ev       : size-by-nev matrix, returns ev*X (X eigenvector matrix)
 */
inline int eig_tridi(int nev,
					 int size, 
					 double *maindiag, 
					 double *subdiag,
					 double *ev)
{
  int n = nev;
  int ierr=0;
  iqlvms (&n, &size, maindiag, subdiag, ev, &nev, &ierr);
  return ierr;
};



/*********** last line of heston.h **************/
#endif 
